@@ -1,7 +1,7 @@
 #
 # Run this Tcl script to generate the lang-*.html files.
 #
-set rcsid {$Id: lang.tcl,v 1.114 2006/06/26 10:37:12 drh Exp $}
+set rcsid {$Id: lang.tcl,v 1.122 2007/02/13 02:03:25 drh Exp $}
 source common.tcl
 
 if {[llength $argv]>0} {
@@ -558,12 +558,24 @@ then the B-Tree key will be a automatically generated integer.  The
 B-Tree key for a row can always be accessed using one of the
 special names "<b>ROWID</b>", "<b>OID</b>", or "<b>_ROWID_</b>".
 This is true regardless of whether or not there is an INTEGER
-PRIMARY KEY.  An INTEGER PRIMARY KEY column man also include the
+PRIMARY KEY.  An INTEGER PRIMARY KEY column can also include the
 keyword AUTOINCREMENT.  The AUTOINCREMENT keyword modified the way
 that B-Tree keys are automatically generated.  Additional detail
 on automatic B-Tree key generation is available
 <a href="autoinc.html">separately</a>.</p>
 
+<p>According to the SQL standard, PRIMARY KEY should imply NOT NULL.
+Unfortunately, due to a long-standing coding oversight, this is not 
+the case in SQLite.  SQLite allows NULL values
+in a PRIMARY KEY column.  We could change SQLite to conform to the
+standard (and we might do so in the future), but by the time the
+oversight was discovered, SQLite was in such wide use that we feared
+breaking legacy code if we fixed the problem.  So for now we have
+chosen to contain allowing NULLs in PRIMARY KEY columns.
+Developers should be aware, however, that we may change SQLite to
+conform to the SQL standard in future and should design new programs
+accordingly.</p>
+
 <p>If the "TEMP" or "TEMPORARY" keyword occurs in between "CREATE"
 and "TABLE" then the table that is created is only visible
 within that same database connection
@@ -626,13 +638,13 @@ statement.  </p>
 Section {CREATE TRIGGER} createtrigger
 
 Syntax {sql-statement} {
-CREATE [TEMP | TEMPORARY] TRIGGER <trigger-name> [ BEFORE | AFTER ]
+CREATE [TEMP | TEMPORARY] TRIGGER [IF NOT EXISTS] <trigger-name> [ BEFORE | AFTER ]
 <database-event> ON [<database-name> .] <table-name>
 <trigger-action>
 }
 
 Syntax {sql-statement} {
-CREATE [TEMP | TEMPORARY] TRIGGER <trigger-name> INSTEAD OF
+CREATE [TEMP | TEMPORARY] TRIGGER [IF NOT EXISTS] <trigger-name> INSTEAD OF
 <database-event> ON [<database-name> .] <view-name>
 <trigger-action>
 }
@@ -782,7 +794,7 @@ statement.</p>
 Section {CREATE VIEW} {createview}
 
 Syntax {sql-command} {
-CREATE [TEMP | TEMPORARY] VIEW [<database-name>.] <view-name> AS <select-statement>
+CREATE [TEMP | TEMPORARY] VIEW [IF NOT EXISTS] [<database-name>.] <view-name> AS <select-statement>
 }
 
 puts {
@@ -929,7 +941,7 @@ result if the table does not exist.</p>
 
 Section {DROP TRIGGER} droptrigger
 Syntax {sql-statement} {
-DROP TRIGGER [<database-name> .] <trigger-name>
+DROP TRIGGER [IF EXISTS] [<database-name> .] <trigger-name>
 }
 puts { 
 <p>The DROP TRIGGER statement removes a trigger created by the 
@@ -942,7 +954,7 @@ dropped when the associated table is dropped.</p>
 Section {DROP VIEW} dropview
 
 Syntax {sql-command} {
-DROP VIEW <view-name>
+DROP VIEW [IF EXISTS] <view-name>
 }
 
 puts {
@@ -995,7 +1007,8 @@ Syntax {expr} {
 <expr> [NOT] IN [<database-name> .] <table-name> |
 [EXISTS] ( <select-statement> ) |
 CASE [<expr>] LP WHEN <expr> THEN <expr> RPPLUS [ELSE <expr>] END |
-CAST ( <expr> AS <type> )
+CAST ( <expr> AS <type> ) |
+<expr> COLLATE <collation-name>
 } {like-op} {
 LIKE | GLOB | REGEXP | MATCH
 }
@@ -1020,12 +1033,17 @@ AND
 OR</font>
 </pre></blockquote>
 
-<p>Supported unary operators are these:</p>
+<p>Supported unary prefix operators are these:</p>
 
 <blockquote><pre>
 <font color="#2c2cf0"><big>-    +    !    ~    NOT</big></font>
 </pre></blockquote>
 
+<p>The COLLATE operator can be thought of as a unary postfix
+operator.  The COLLATE operator has the highest precedence.
+It always binds more tightly than any prefix unary operator or
+any binary operator.</p>
+
 <p>The unary operator [Operator +] is a no-op.  It can be applied
 to strings, numbers, or blobs and it always gives as its result the
 value of the operand.</p>
@@ -1253,8 +1271,9 @@ all arguments are NULL then NULL is returned.  There must be at least
 </tr>
 
 <tr>
+<td valign="top" align="right">
 <a name="globFunc"></a>
-<td valign="top" align="right">glob(<i>X</i>,<i>Y</i>)</td>
+glob(<i>X</i>,<i>Y</i>)</td>
 <td valign="top">This function is used to implement the
 "<b>X GLOB Y</b>" syntax of SQLite.  The
 <a href="capi3ref.html#sqlite3_create_function">sqlite3_create_function()</a> 
@@ -1270,6 +1289,14 @@ both arguments are NULL then NULL is returned. This behaves the same as
 <b>coalesce()</b> above.</td>
 </tr>
 
+<tr>
+<td valign="top" align="right">
+<a name="hexFunc">
+hex(<i>X</i>)</td>
+<td valign="top">The argument is interpreted as a BLOB.  The result
+is a hexadecimal rendering of the content of that blob.</td>
+</tr>
+
 <tr>
 <td valign="top" align="right">last_insert_rowid()</td>
 <td valign="top">Return the ROWID of the last row insert from this
@@ -1285,8 +1312,9 @@ characters is returned, not the number of bytes.</td>
 </tr>
 
 <tr>
+<td valign="top" align="right">
 <a name="likeFunc"></a>
-<td valign="top" align="right">like(<i>X</i>,<i>Y</i> [,<i>Z</i>])</td>
+like(<i>X</i>,<i>Y</i> [,<i>Z</i>])</td>
 <td valign="top">
 This function is used to implement the "<b>X LIKE Y [ESCAPE Z]</b>"
 syntax of SQL. If the optional ESCAPE clause is present, then the
@@ -1361,6 +1389,14 @@ is also useful when writing triggers to implement undo/redo functionality.
 between -9223372036854775808 and +9223372036854775807.</td>
 </tr>
 
+<tr>
+<td valign="top" align="right">
+<a name="randomblobFunc">
+randomblob(<i>N</i>)</td>
+<td valign="top">Return a <i>N</i>-byte blob containing pseudo-random bytes.
+<i>N</i> should be a postive integer.</td>
+</tr>
+
 <tr>
 <td valign="top" align="right">round(<i>X</i>)<br>round(<i>X</i>,<i>Y</i>)</td>
 <td valign="top">Round off the number <i>X</i> to <i>Y</i> digits to the
@@ -1515,8 +1551,7 @@ SELECT must exactly match the number of columns in the table if
 no column list is specified, or it must match the number of columns
 name in the column list.  A new entry is made in the table
 for every row of the SELECT result.  The SELECT may be simple
-or compound.  If the SELECT statement has an ORDER BY clause,
-the ORDER BY is ignored.</p>
+or compound.</p>
 
 <p>The optional conflict-clause allows the specification of an alternative
 constraint conflict resolution algorithm to use during this one command.
